@@ -5,6 +5,7 @@ use crate::module_loader::{
 use std::borrow::Cow;
 use std::collections::hash_map::Entry;
 use std::collections::{HashMap, HashSet};
+use std::fmt::{Debug, Display, Formatter};
 use std::io::Cursor;
 use std::path::Path;
 use std::rc::Rc;
@@ -14,7 +15,7 @@ use deno_runtime::deno_core::anyhow::bail;
 use deno_runtime::deno_core::error::AnyError;
 use deno_runtime::deno_core::{serde_v8, v8, Extension};
 
-use deno_core::{op, ModuleCode, Op};
+use deno_core::{op2, ModuleCode, Op};
 use deno_runtime::deno_broadcast_channel::InMemoryBroadcastChannel;
 use deno_runtime::deno_core;
 use deno_runtime::deno_web::BlobStore;
@@ -24,6 +25,7 @@ use deno_runtime::worker::WorkerOptions;
 
 use deno_runtime::deno_fs::RealFs;
 use std::panic;
+use std::str::FromStr;
 use std::thread;
 use std::thread::JoinHandle;
 
@@ -39,6 +41,8 @@ use image::io::Reader as ImageReader;
 
 use crate::text::{op_text_width, FONT_DB, USVG_OPTIONS};
 
+deno_core::extension!(vl_convert_converter_runtime, ops = [op_get_json_arg]);
+
 lazy_static! {
     pub static ref TOKIO_RUNTIME: tokio::runtime::Runtime =
         tokio::runtime::Builder::new_current_thread()
@@ -57,9 +61,14 @@ pub struct VgOpts {
 }
 
 impl VgOpts {
-    pub fn to_embed_opts(&self) -> Result<serde_json::Value, AnyError> {
+    pub fn to_embed_opts(&self, renderer: Renderer) -> Result<serde_json::Value, AnyError> {
         let mut opts_map = serde_json::Map::new();
 
+        opts_map.insert(
+            "renderer".to_string(),
+            serde_json::Value::String(renderer.to_string()),
+        );
+
         if let Some(format_locale) = &self.format_locale {
             opts_map.insert("formatLocale".to_string(), format_locale.as_object()?);
         }
@@ -114,6 +123,37 @@ impl TimeFormatLocale {
     }
 }
 
+#[derive(Debug, Clone, Copy)]
+pub enum Renderer {
+    Svg,
+    Canvas,
+    Hybrid,
+}
+
+impl Display for Renderer {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        let r = match self {
+            Renderer::Svg => "svg",
+            Renderer::Canvas => "canvas",
+            Renderer::Hybrid => "hybrid",
+        };
+        std::fmt::Display::fmt(r, f)
+    }
+}
+
+impl FromStr for Renderer {
+    type Err = AnyError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        Ok(match s.to_ascii_lowercase().as_str() {
+            "svg" => Self::Svg,
+            "canvas" => Self::Canvas,
+            "hybrid" => Self::Hybrid,
+            _ => return Err(anyhow!("Unsupported renderer: {}", s)),
+        })
+    }
+}
+
 #[derive(Debug, Clone, Default)]
 pub struct VlOpts {
     pub config: Option<serde_json::Value>,
@@ -126,9 +166,14 @@ pub struct VlOpts {
 }
 
 impl VlOpts {
-    pub fn to_embed_opts(&self) -> Result<serde_json::Value, AnyError> {
+    pub fn to_embed_opts(&self, renderer: Renderer) -> Result<serde_json::Value, AnyError> {
         let mut opts_map = serde_json::Map::new();
 
+        opts_map.insert(
+            "renderer".to_string(),
+            serde_json::Value::String(renderer.to_string()),
+        );
+
         if let Some(theme) = &self.theme {
             opts_map.insert(
                 "theme".to_string(),
@@ -180,7 +225,8 @@ fn set_json_arg(arg: serde_json::Value) -> Result<i32, AnyError> {
     Ok(id)
 }
 
-#[op]
+#[op2]
+#[string]
 fn op_get_json_arg(arg_id: i32) -> Result<String, AnyError> {
     match JSON_ARGS.lock() {
         Ok(mut guard) => {
@@ -330,9 +376,20 @@ function vegaToSvg(vgSpec, allowedBaseUrls, formatLocale, timeFormatLocale, erro
         vega.timeFormatLocale(timeFormatLocale);
     }
     let view = vegaToView(vgSpec, allowedBaseUrls, errors);
-    let svgPromise = view.toSVG().finally(() => {
-        view.finalize();
-        vega.resetDefaultLocale();
+    let svgPromise = view.runAsync().then(() => {
+        try {
+            // Workaround for https://github.com/vega/vega/issues/3481
+            view.signal("geo_interval_init_tick", {});
+        } catch (e) {
+            // No geo_interval_init_tick signal
+        }
+    }).then(() => {
+        return view.runAsync().then(
+            () => view.toSVG()
+        ).finally(() => {
+            view.finalize();
+            vega.resetDefaultLocale();
+        })
     });
     return svgPromise
 }
@@ -346,10 +403,19 @@ function vegaToScenegraph(vgSpec, allowedBaseUrls, formatLocale, timeFormatLocal
     }
     let view = vegaToView(vgSpec, allowedBaseUrls, errors);
     let scenegraphPromise = view.runAsync().then(() => {
-        return JSON.parse(JSON.parse(vega.sceneToJSON(view.scenegraph())));
-    }).finally(() => {
-        view.finalize();
-        vega.resetDefaultLocale();
+        try {
+            // Workaround for https://github.com/vega/vega/issues/3481
+            view.signal("geo_interval_init_tick", {});
+        } catch (e) {
+            // No geo_interval_init_tick signal
+        }
+    }).then(() => {
+        return view.runAsync().then(
+            () => JSON.parse(JSON.parse(vega.sceneToJSON(view.scenegraph())))
+        ).finally(() => {
+            view.finalize();
+            vega.resetDefaultLocale();
+        })
     });
     return scenegraphPromise
 }
@@ -448,6 +514,7 @@ function vegaLiteToScenegraph_{ver_name}(vlSpec, config, theme, warnings, allowe
             bootstrap: Default::default(),
             extensions: vec![ext],
             startup_snapshot: None,
+            skip_op_registration: false,
             create_params: None,
             unsafely_ignore_certificate_errors: None,
             root_cert_store_provider: None,
@@ -470,6 +537,7 @@ function vegaLiteToScenegraph_{ver_name}(vlSpec, config, theme, warnings, allowe
             should_wait_for_inspector_session: false,
             fs: Arc::new(RealFs),
             feature_checker: Arc::new(Default::default()),
+            strace_ops: None,
         };
 
         let main_module =
@@ -1282,9 +1350,10 @@ impl VlConverter {
         vl_spec: serde_json::Value,
         vl_opts: VlOpts,
         bundle: bool,
+        renderer: Renderer,
     ) -> Result<String, AnyError> {
         let vl_version = vl_opts.vl_version;
-        let code = get_vega_or_vegalite_script(vl_spec, vl_opts.to_embed_opts()?)?;
+        let code = get_vega_or_vegalite_script(vl_spec, vl_opts.to_embed_opts(renderer)?)?;
         self.build_html(&code, vl_version, bundle).await
     }
 
@@ -1293,8 +1362,9 @@ impl VlConverter {
         vg_spec: serde_json::Value,
         vg_opts: VgOpts,
         bundle: bool,
+        renderer: Renderer,
     ) -> Result<String, AnyError> {
-        let code = get_vega_or_vegalite_script(vg_spec, vg_opts.to_embed_opts()?)?;
+        let code = get_vega_or_vegalite_script(vg_spec, vg_opts.to_embed_opts(renderer)?)?;
         self.build_html(&code, Default::default(), bundle).await
     }
 
@@ -1542,7 +1612,7 @@ mod tests {
             .vegalite_to_vega(
                 vl_spec,
                 VlOpts {
-                    vl_version: VlVersion::v4_17,
+                    vl_version: VlVersion::v5_16,
                     ..Default::default()
                 },
             )
@@ -1569,7 +1639,7 @@ mod tests {
             .vegalite_to_vega(
                 vl_spec.clone(),
                 VlOpts {
-                    vl_version: VlVersion::v4_17,
+                    vl_version: VlVersion::v5_16,
                     ..Default::default()
                 },
             )
