@@ -317,7 +317,6 @@ mod test_vegalite_to_vega {
     #[rstest]
     fn test(
         #[values(
-            VlVersion::v4_17,
             VlVersion::v5_8,
             VlVersion::v5_9,
             VlVersion::v5_10,
@@ -357,7 +356,7 @@ mod test_vegalite_to_vega {
 mod test_vegalite_to_html_no_bundle {
     use crate::*;
     use futures::executor::block_on;
-    use vl_convert_rs::converter::VlOpts;
+    use vl_convert_rs::converter::{Renderer, VlOpts};
     use vl_convert_rs::VlConverter;
 
     #[rstest]
@@ -387,7 +386,7 @@ mod test_vegalite_to_html_no_bundle {
         let mut converter = VlConverter::new();
 
         let html_result = block_on(
-            converter.vegalite_to_html(vl_spec, VlOpts{vl_version, ..Default::default()}, false)
+            converter.vegalite_to_html(vl_spec, VlOpts{vl_version, ..Default::default()}, false, Renderer::Canvas)
         ).unwrap();
 
         // Check for expected patterns
@@ -405,7 +404,7 @@ mod test_vegalite_to_html_no_bundle {
 mod test_vegalite_to_html_bundle {
     use crate::*;
     use futures::executor::block_on;
-    use vl_convert_rs::converter::VlOpts;
+    use vl_convert_rs::converter::{Renderer, VlOpts};
     use vl_convert_rs::VlConverter;
 
     #[rstest]
@@ -435,7 +434,7 @@ mod test_vegalite_to_html_bundle {
         let mut converter = VlConverter::new();
 
         let html_result = block_on(
-            converter.vegalite_to_html(vl_spec, VlOpts{vl_version, ..Default::default()}, true)
+            converter.vegalite_to_html(vl_spec, VlOpts{vl_version, ..Default::default()}, true, Renderer::Svg)
         ).unwrap();
 
         // Check for expected patterns
@@ -641,6 +640,8 @@ mod test_png_no_theme {
         case("no_text_in_font_metrics", 1.0),
         case("custom_projection", 1.0),
         case("long_legend_label", 1.0),
+        case("quakes_initial_selection", 1.0),
+        case("geoScale", 1.0),
     )]
     fn test(
         name: &str,
